#include <cerrno>
#include <cstring>
#include <sstream>
#include <iostream>
#include "xunix.h"

xunix::exception::exception(const std::string& prefix) : std::exception()
{
  init(prefix);
}

xunix::exception::~exception() throw()
{
  // empty
}

const char* xunix::exception::what() const throw()
{
  return _what.c_str();
}

int xunix::exception::error_number() const throw()
{
  return _error_number;
}

void xunix::exception::init(const std::string& prefix)
{
  // Save errno asap.
  _error_number = errno;
  std::ostringstream ostrm;
  if( prefix.length() > 0 ) {
    ostrm << prefix << ": ";
  }
  ostrm << strerror(_error_number);
  _what += ostrm.str();
}

void xunix::sigaction(int signum,
                     const struct sigaction* act,
                     struct sigaction* oldact)
  throw(xunix::exception)
{
  if( ::sigaction(signum, act, oldact) == -1 ) {
    std::ostringstream ostrm;
    ostrm << "sigaction(" << signum << ")";
    throw(xunix::exception(ostrm.str()));
  }
}

void xunix::sigemptyset(sigset_t* sigset)
{
  if( ::sigemptyset(sigset) == - 1 ) {
    throw( xunix::exception("sigemptyset") );
  }
}

void xunix::sigfillset(sigset_t* sigset)
{
  if( ::sigfillset(sigset) == -1 ) {
    throw( xunix::exception("sigfillset") );
  }
}

void xunix::sigaddset(sigset_t* sigset, int signum)
{
  if( ::sigaddset(sigset, signum) == -1 ) {
    throw( xunix::exception("sigaddset") );
  }
}

void xunix::sigdelset(sigset_t* sigset, int signum)
{
  if( ::sigdelset(sigset, signum) == -1 ) {
    throw( xunix::exception("sigdelset") );
  }
}

int xunix::sigismember(const sigset_t* sigset, int signum)
{
  int rv = ::sigismember(sigset, signum);
  if( rv == -1 ) {
    throw( xunix::exception("sigismember") );
  }
  return rv;
}

void xunix::pause(void)
{
  ::pause();
}

void xunix::pipe(int fds[2])
{
  if( ::pipe(fds) == -1 ) {
    throw(xunix::exception("pipe"));
  }
}

int xunix::select(int n,
                  fd_set* rset,
                  fd_set* wset,
                  fd_set* eset,
                  struct timeval* t)
{
  int rv = ::select(n, rset, wset, eset, t);
  if( rv == -1 ) {
    throw(xunix::exception("select"));
  }
  return rv;
}

ssize_t xunix::read(int fd, void* buf, size_t count)
{
  ssize_t rv = ::read(fd, buf, count);
  if( rv == -1 ) {
    std::ostringstream ostrm;
    ostrm << "read(" << fd << ")";
    throw(xunix::exception(ostrm.str()));
  }
  return rv;
}

ssize_t xunix::write(int fd, const void* buf, size_t count)
{
  ssize_t rv = ::write(fd, buf, count);
  if( rv == -1 ) {
    std::ostringstream ostrm;
    ostrm << "write(" << fd << ")";
    throw(xunix::exception(ostrm.str()));
  }
  return rv;
}
